@@ -142,6 +142,8 @@ OC.L10N.register(
     "Drag this to your browser bookmarks and click it to quickly bookmark a webpage." : "Drag this to your browser bookmarks and click it to quickly bookmark a webpage.",
     "Add to {instanceName}" : "افزودن به {instanceName}",
     "Need help?" : "کمک لازم دارید ؟",
+    "Created {time} ago" : "Created {time} ago",
+    "Created on {date}" : "Created on {date}",
     "Title" : "عنوان",
     "Link" : "پیوند",
     "Untitled folder" : "Untitled folder",
@@ -151,6 +153,7 @@ OC.L10N.register(
     "Notes" : "یادداشت ها",
     "Notes for this bookmark …" : "Notes for this bookmark …",
     "Archived file" : "Archived file",
+    "Open file" : "باز کردن فایل",
     "Download file" : "دانلود فایل",
     "Open file location" : "Open file location",
     "Link copied" : "پیوند رونوشت شد",
@@ -209,6 +212,8 @@ OC.L10N.register(
     "Failed to load folders" : "بارگیری پوشه ها انجام نشد",
     "Failed to delete folder" : "پوشه حذف نشد",
     "Failed to create folder" : "پوشه ایجاد نشد",
+    "Failed to move parts of selection" : "جابجایی بخش های انتخاب انجام نشد",
+    "Failed to delete parts of selection" : "حذف قسمتهای انتخاب انجام نشد",
     "Failed to fetch bookmarks." : "واکشی نشانک ها انجام نشد.",
     "Failed to load setting {key}" : "شکست در بار کردن تنظیم {key}",
     "Failed to update share {shareId}" : "Failed to update share {shareId}",
