@@ -111,6 +111,9 @@ OC.L10N.register(
     "No bookmarks here" : "ブックマークがありません",
     "Archive path" : "アーカイブパス",
     "Please select \"Add to home screen\" in your browser menu" : "お使いのブラウザーで「ホーム画面に追加」を選択してください",
+    "Context Chat integration" : "コンテキストチャットの統合",
+    "Context chat is currently not installed, but is required for this feature." : "コンテキスト チャットは現在インストールされていませんが、この機能には必要です。",
+    "Enable Context Chat integration" : "コンテキストチャットの統合を有効にする",
     "Client apps" : "クライアントアプリ",
     "Also check out the collection of client apps that integrate with this app: " : "このアプリと連携するクライアントアプリもご確認ください：",
     "Install web app" : "Webアプリをインストール",
@@ -128,6 +131,7 @@ OC.L10N.register(
     "Notes" : "ノート",
     "Notes for this bookmark …" : "このブックマークのメモ…",
     "Archived file" : "アーカイブ済みファイル",
+    "Open file" : "ファイルを開く",
     "Download file" : "ダウンロードファイル",
     "Open file location" : "ファイルの場所を開く",
     "Link copied" : "リンクをコピーしました",
