@@ -10,6 +10,7 @@ OC.L10N.register(
     "New folder" : "Ny mappe",
     "Search" : "Søk",
     "Empty trash bin" : "Tøm søppel korg",
+    "_%n folder_::_%n folders_" : ["%n mappe","%n mapper"],
     "Choose folder" : "Vel mappe",
     "Cancel" : "Avbryt",
     "Shared with you" : "Shared with you",
@@ -20,12 +21,13 @@ OC.L10N.register(
     "Link" : "Lenkje",
     "Tags" : "Emneord",
     "Notes" : "Notat",
+    "Open file" : "Opne fil",
+    "Link copied" : "Lenkje kopiert",
     "Owner" : "Owner",
     "Sharing" : "Deling",
     "Share link" : "Del lenkje",
     "Allow editing" : "Tillat endring",
     "Allow resharing" : "Tillat vidaredeling",
-    "Link copied" : "Lenkje kopiert",
     "Privacy" : "Personvern",
     "Folder" : "Mappe",
     "Save" : "Lagre"
