@@ -3,40 +3,39 @@ OC.L10N.register(
     {
     "Details" : "Detajet",
     "Rename" : "Rename",
+    "Copy link" : "Kopjo linkun",
     "Move" : "Zhvendos",
-    "Delete" : "Fshij",
-    "Search" : "Kërko",
+    "Select all" : "Zgjidhi të gjitha",
+    "Go back" : "Shko prapa",
     "New" : "I ri",
     "New folder" : "Dosje e re",
-    "Grid view" : "Pamje galeri",
-    "List view" : "Pamje listë",
-    "Select all" : "Zgjidhi të gjitha",
+    "Search" : "Kërko",
+    "_%n folder_::_%n folders_" : ["%n dosje","%n dosje"],
+    "Rename folder" : "Riemërtoni dosjen",
+    "Choose folder" : "Zgjidhni dosjen",
     "Submit" : "Dërgo",
     "Cancel" : "Anullo",
     "Recent" : "Së fundmi",
+    "Shared with you" : "E ndarë me ju",
+    "Files" : "Skedarët",
+    "Delete" : "Fshij",
     "Untagged" : "E paetiketuar",
-    "Import" : "Import",
-    "Export" : "Eksport",
-    "Recently added" : "Shtuar së fundmi",
-    "Alphabetically" : "Alfabetikisht",
-    "Most visited" : "Vizituar më shpesh",
-    "Last modified" : "Ndryshuar së fundmi",
     "Bookmarklet" : "Libër Shënues",
-    "Clear data" : "Pastroji të dhënat",
+    "Need help?" : "Ju duhet ndihmë?",
     "Title" : "Titulli",
     "Link" : "Lidhje",
     "Tags" : "Etiketat",
-    "Attachments" : "Bashkangjitjet",
     "Notes" : "Shënime",
+    "Open file" : "Hap skedarin",
+    "Download file" : "Shkarko skedarin",
+    "Link copied" : "Linku u kopjua",
     "Owner" : "Zotëruesi",
     "Sharing" : "Ndarje",
     "Share link" : "Share link",
-    "Copy link" : "Kopjo linkun",
     "Allow editing" : "Lejo redaktimin",
-    "Link copied" : "Linku u kopjua",
-    "Select" : "Përzgjidh",
-    "Previews" : "Parapamjet",
+    "Allow resharing" : "Lejo rindarje",
     "Privacy" : "Privatësia",
+    "Previews" : "Parapamjet",
     "Folder" : "Skedari",
     "Save" : "Ruaj"
 },
