@@ -109,6 +109,7 @@ class BookmarkMapper extends QBMapper {
 			->select('*')
 			->from('bookmarks')
 			->where($qb->expr()->eq('user_id', $qb->createParameter('user_id')))
+			->andWhere($qb->expr()->eq('url_hash', $qb->createParameter('url_hash')))
 			->andWhere($qb->expr()->eq('url', $qb->createParameter('url')));
 		return $qb;
 	}
@@ -132,7 +133,8 @@ class BookmarkMapper extends QBMapper {
 		$qb = $this->findByUrlQuery;
 		$qb->setParameters([
 			'user_id' => $userId,
-			'url' => $url
+			'url' => $url,
+			'url_hash' => hash('xxh128', $url),
 		]);
 		return $this->findEntity($qb);
 	}
@@ -141,17 +143,19 @@ class BookmarkMapper extends QBMapper {
 	 * @param string $userId
 	 * @throws DoesNotExistException
 	 * @throws MultipleObjectsReturnedException
+	 * @throws Exception
 	 */
 	public function deleteAll(string $userId): void {
 		$qb = $this->db->getQueryBuilder();
 		$qb->select('b.id')
 			->from('bookmarks', 'b')
 			->where($qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)));
-		$orphanedBookmarks = $qb->execute();
-		while ($bookmark = $orphanedBookmarks->fetchColumn()) {
+		$result = $qb->executeQuery();
+		while ($bookmark = $result->fetchOne()) {
 			$bm = $this->find($bookmark);
 			$this->delete($bm);
 		}
+		$result->closeCursor();
 	}
 
 	/**
@@ -265,8 +269,6 @@ class BookmarkMapper extends QBMapper {
 	protected function getIteratorWithRawQuery(string $query, array $params, array $types): \Generator {
 		$cursor = $this->db->executeQuery($query, $params, $types);
 
-		$entities = [];
-
 		while ($row = $cursor->fetch()) {
 			yield $this->mapRowToEntity($row);
 		}
@@ -401,6 +403,7 @@ class BookmarkMapper extends QBMapper {
 	private function _filterUrl(IQueryBuilder $qb, QueryParameters $params): void {
 		if (($url = $params->getUrl()) !== null) {
 			$normalized = $this->urlNormalizer->normalize($url);
+			$qb->andWhere($qb->expr()->eq('b.url_hash', $qb->createPositionalParameter(hash('xxh128', $normalized))));
 			$qb->andWhere($qb->expr()->eq('b.url', $qb->createPositionalParameter($normalized)));
 		}
 	}
@@ -528,6 +531,38 @@ class BookmarkMapper extends QBMapper {
 		}
 	}
 
+	/**
+	 * @param string $userId
+	 * @return int
+	 * @throws Exception
+	 */
+	public function countDeleted(string $userId): int {
+		$qb = $this->db->getQueryBuilder();
+		$qb->selectAlias($qb->func()->count('b.id'), 'count');
+		$qb
+			->from('bookmarks', 'b')
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK))
+			->where($qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)))
+			->andWhere($qb->expr()->isNotNull('tr.soft_deleted_at'));
+		$result = $qb->executeQuery();
+		$userOwnerDeletedCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
+
+		$qb = $this->db->getQueryBuilder();
+		$qb->selectAlias($qb->func()->count('b.id'), 'count');
+		$qb
+			->from('bookmarks', 'b')
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK))
+			->innerJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
+			->where($qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId)))
+			->andWhere($qb->expr()->isNotNull('tr.soft_deleted_at'));
+		$result = $qb->executeQuery();
+		$foreignDeletedCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
+
+		return $userOwnerDeletedCount + $foreignDeletedCount;
+	}
+
 	/**
 	 * @param string $userId
 	 * @return int
@@ -535,23 +570,28 @@ class BookmarkMapper extends QBMapper {
 	public function countArchived(string $userId): int {
 		$qb = $this->db->getQueryBuilder();
 		$qb->selectAlias($qb->func()->count('b.id'), 'count');
-
 		$qb
 			->from('bookmarks', 'b')
-			->leftJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
-			->leftJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
-			->where(
-				$qb->expr()->andX(
-					$qb->expr()->orX(
-						$qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)),
-						$qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId))
-					),
-					$qb->expr()->in('b.user_id', array_map([$qb, 'createPositionalParameter'], array_merge($this->_findSharersFor($userId), [$userId])))
-				)
-			)
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
+			->where($qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)))
 			->andWhere($qb->expr()->isNotNull('b.archived_file'));
+		$result = $qb->executeQuery();
+		$userOwnerArchivedCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
 
-		return $qb->execute()->fetch(PDO::FETCH_COLUMN);
+		$qb = $this->db->getQueryBuilder();
+		$qb->selectAlias($qb->func()->count('b.id'), 'count');
+		$qb
+			->from('bookmarks', 'b')
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
+			->innerJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
+			->where($qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId)))
+			->andWhere($qb->expr()->isNotNull('b.archived_file'));
+		$result = $qb->executeQuery();
+		$foreignArchivedCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
+
+		return $userOwnerArchivedCount + $foreignArchivedCount;
 	}
 
 	/**
@@ -562,22 +602,28 @@ class BookmarkMapper extends QBMapper {
 	public function countAllClicks(string $userId): int {
 		$qb = $this->db->getQueryBuilder();
 		$qb->selectAlias($qb->func()->sum('b.clickcount'), 'count');
-
 		$qb
 			->from('bookmarks', 'b')
-			->leftJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
-			->leftJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
-			->where(
-				$qb->expr()->andX(
-					$qb->expr()->orX(
-						$qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)),
-						$qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId))
-					),
-					$qb->expr()->in('b.user_id', array_map([$qb, 'createPositionalParameter'], array_merge($this->_findSharersFor($userId), [$userId])))
-				)
-			);
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
+			->where($qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)))
+			->andWhere($qb->expr()->neq('b.clickcount', $qb->createPositionalParameter(0, IQueryBuilder::PARAM_INT)));
+		$result = $qb->executeQuery();
+		$userOwnerClickCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
 
-		return $qb->execute()->fetch(PDO::FETCH_COLUMN) ?? 0;
+		$qb = $this->db->getQueryBuilder();
+		$qb->selectAlias($qb->func()->sum('b.clickcount'), 'count');
+		$qb
+			->from('bookmarks', 'b')
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
+			->innerJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
+			->where($qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId)))
+			->andWhere($qb->expr()->neq('b.clickcount', $qb->createPositionalParameter(0, IQueryBuilder::PARAM_INT)));
+		$result = $qb->executeQuery();
+		$foreignClickCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
+
+		return $userOwnerClickCount + $foreignClickCount;
 	}
 
 	/**
@@ -588,23 +634,28 @@ class BookmarkMapper extends QBMapper {
 	public function countWithClicks(string $userId): int {
 		$qb = $this->db->getQueryBuilder();
 		$qb->selectAlias($qb->func()->count('b.id'), 'count');
-
 		$qb
 			->from('bookmarks', 'b')
-			->leftJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
-			->leftJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
-			->where(
-				$qb->expr()->andX(
-					$qb->expr()->orX(
-						$qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)),
-						$qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId))
-					),
-					$qb->expr()->in('b.user_id', array_map([$qb, 'createPositionalParameter'], array_merge($this->_findSharersFor($userId), [$userId])))
-				)
-			)
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
+			->where($qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)))
 			->andWhere($qb->expr()->neq('b.clickcount', $qb->createPositionalParameter(0, IQueryBuilder::PARAM_INT)));
+		$result = $qb->executeQuery();
+		$userOwnerWithClicksCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
 
-		return $qb->execute()->fetch(PDO::FETCH_COLUMN);
+		$qb = $this->db->getQueryBuilder();
+		$qb->selectAlias($qb->func()->count('b.id'), 'count');
+		$qb
+			->from('bookmarks', 'b')
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
+			->innerJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
+			->where($qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId)))
+			->andWhere($qb->expr()->neq('b.clickcount', $qb->createPositionalParameter(0, IQueryBuilder::PARAM_INT)));
+		$result = $qb->executeQuery();
+		$foreignWithClicksCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
+
+		return $userOwnerWithClicksCount + $foreignWithClicksCount;
 	}
 
 	/**
@@ -615,55 +666,74 @@ class BookmarkMapper extends QBMapper {
 	public function countUnavailable(string $userId): int {
 		$qb = $this->db->getQueryBuilder();
 		$qb->selectAlias($qb->func()->count('b.id'), 'count');
-
 		$qb
 			->from('bookmarks', 'b')
-			->leftJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
-			->leftJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
-			->where(
-				$qb->expr()->andX(
-					$qb->expr()->orX(
-						$qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)),
-						$qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId))
-					),
-					$qb->expr()->in('b.user_id', array_map([$qb, 'createPositionalParameter'], array_merge($this->_findSharersFor($userId), [$userId])))
-				)
-			)
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
+			->where($qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)))
 			->andWhere($qb->expr()->eq('b.available', $qb->createPositionalParameter(false, IQueryBuilder::PARAM_BOOL)));
+		$result = $qb->executeQuery();
+		$userOwnerUnavailableCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
 
-		return $qb->execute()->fetch(PDO::FETCH_COLUMN);
+		$qb = $this->db->getQueryBuilder();
+		$qb->selectAlias($qb->func()->count('b.id'), 'count');
+		$qb
+			->from('bookmarks', 'b')
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
+			->innerJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
+			->where($qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId)))
+			->andWhere($qb->expr()->eq('b.available', $qb->createPositionalParameter(false, IQueryBuilder::PARAM_BOOL)));
+		$result = $qb->executeQuery();
+		$foreignUnavailableCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
+
+		return $userOwnerUnavailableCount + $foreignUnavailableCount;
 	}
 
 	/**
 	 * @param string $userId
 	 * @return int
+	 * @throws Exception
 	 */
 	public function countDuplicated(string $userId): int {
 		$qb = $this->db->getQueryBuilder();
 		$qb->selectDistinct($qb->func()->count('b.id'));
-
 		$qb
 			->from('bookmarks', 'b')
-			->leftJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
-			->leftJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
-			->where(
-				$qb->expr()->andX(
-					$qb->expr()->orX(
-						$qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)),
-						$qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId))
-					),
-					$qb->expr()->in('b.user_id', array_map([$qb, 'createPositionalParameter'], array_merge($this->_findSharersFor($userId), [$userId]))),
-				)
-			);
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
+			->where($qb->expr()->eq('b.user_id', $qb->createPositionalParameter($userId)));
 		$subQuery = $this->db->getQueryBuilder();
 		$subQuery->select('trdup.parent_folder')
 			->from('bookmarks_tree', 'trdup')
 			->where($subQuery->expr()->eq('b.id', 'trdup.id'))
 			->andWhere($subQuery->expr()->neq('trdup.parent_folder', 'tr.parent_folder'))
-			->andWhere($subQuery->expr()->eq('trdup.type', $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK)));
+			->andWhere($subQuery->expr()->eq('trdup.type', $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK)))
+			->andWhere($subQuery->expr()->isNull('trdup.soft_deleted_at'));
 		$qb->andWhere($qb->createFunction('EXISTS(' . $subQuery->getSQL() . ')'));
+		$result = $qb->executeQuery();
+		$userOwnerDuplicatesCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
 
-		return $qb->execute()->fetch(PDO::FETCH_COLUMN);
+		$qb = $this->db->getQueryBuilder();
+		$qb->selectDistinct($qb->func()->count('b.id'));
+		$qb
+			->from('bookmarks', 'b')
+			->innerJoin('b', 'bookmarks_tree', 'tr', 'b.id = tr.id AND tr.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tr.soft_deleted_at is NULL')
+			->innerJoin('tr', 'bookmarks_shared_folders', 'sf', $qb->expr()->eq('tr.parent_folder', 'sf.folder_id'))
+			->where($qb->expr()->eq('sf.user_id', $qb->createPositionalParameter($userId)));
+		$subQuery = $this->db->getQueryBuilder();
+		$subQuery->select('trdup.parent_folder')
+			->from('bookmarks_tree', 'trdup')
+			->where($subQuery->expr()->eq('b.id', 'trdup.id'))
+			->andWhere($subQuery->expr()->neq('trdup.parent_folder', 'tr.parent_folder'))
+			->andWhere($subQuery->expr()->eq('trdup.type', $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK)))
+			->andWhere($subQuery->expr()->isNull('trdup.soft_deleted_at'));
+		$qb->andWhere($qb->createFunction('EXISTS(' . $subQuery->getSQL() . ')'));
+		$result = $qb->executeQuery();
+		$foreignDuplicatesCount = $result->fetch(PDO::FETCH_COLUMN);
+		$result->closeCursor();
+
+		return $userOwnerDuplicatesCount + $foreignDuplicatesCount;
 	}
 
 	/**
@@ -704,7 +774,7 @@ class BookmarkMapper extends QBMapper {
 
 		$qb
 			->from('*PREFIX*bookmarks', 'b')
-			->join('b', 'folder_tree', 'tree', 'tree.item_id = b.id AND tree.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tree.soft_deleted_at is NULL');
+			->innerJoin('b', 'folder_tree', 'tree', 'tree.item_id = b.id AND tree.type = ' . $qb->createPositionalParameter(TreeMapper::TYPE_BOOKMARK) . ' AND tree.soft_deleted_at is NULL');
 
 
 		$this->_filterUrl($qb, $queryParams);
@@ -773,7 +843,7 @@ class BookmarkMapper extends QBMapper {
 
 		$qb = $this->deleteTagsQuery;
 		$qb->setParameter('id', $id);
-		$qb->execute();
+		$qb->executeStatement();
 
 		return $returnedEntity;
 	}
@@ -813,6 +883,7 @@ class BookmarkMapper extends QBMapper {
 		if ($entity->isWebLink()) {
 			$entity->setUrl($this->urlNormalizer->normalize($entity->getUrl()));
 		}
+		$entity->setUrlHash(hash('xxh128', $entity->getUrl()));
 
 		if ($entity->getAdded() === null) {
 			$entity->setAdded(time());
@@ -822,16 +893,19 @@ class BookmarkMapper extends QBMapper {
 		$entity->setClickcount(0);
 
 		try {
-			$this->findByUrl($entity->getUserId(), $entity->getUrl());
-		} catch (DoesNotExistException $e) {
 			parent::insert($entity);
 			$this->eventDispatcher->dispatchTyped(new InsertEvent('bookmark', $entity->getId()));
 			return $entity;
-		} catch (MultipleObjectsReturnedException $e) {
-			// noop
+		} catch (Exception $e) {
+			if ($e->getReason() === Exception::REASON_UNIQUE_CONSTRAINT_VIOLATION) {
+				if ($this->db->inTransaction()) {
+					$this->db->commit();
+					$this->db->beginTransaction();
+				}
+				throw new AlreadyExistsError('A bookmark with this URL already exists');
+			}
+			throw $e;
 		}
-
-		throw new AlreadyExistsError('A bookmark with this URL already exists');
 	}
 
 	/**
@@ -846,19 +920,20 @@ class BookmarkMapper extends QBMapper {
 		try {
 			$newEntity = $this->insert($entity);
 		} catch (AlreadyExistsError $e) {
-			$bookmark = $this->findByUrl($entity->getUserId(), $entity->getUrl());
-			$entity->setId($bookmark->getId());
-			$newEntity = $this->update($entity);
+			try {
+				$bookmark = $this->findByUrl($entity->getUserId(), $entity->getUrl());
+				$entity->setId($bookmark->getId());
+				$newEntity = $this->update($entity);
+			} catch (DoesNotExistException $e) {
+				$newEntity = $this->insert($entity);
+			}
 		}
 
 		return $newEntity;
 	}
 
 	/**
-	 * @param $userId
-	 * @param string $userId
-	 *
-	 * @return int
+	 * @throws Exception
 	 */
 	public function countBookmarksOfUser(string $userId) : int {
 		$qb = $this->db->getQueryBuilder();
@@ -866,7 +941,13 @@ class BookmarkMapper extends QBMapper {
 			->select($qb->func()->count('id'))
 			->from('bookmarks')
 			->where($qb->expr()->eq('user_id', $qb->createPositionalParameter($userId)));
-		return $qb->execute()->fetch(PDO::FETCH_COLUMN);
+		$result = $qb->executeQuery();
+		$count = $result->fetchOne();
+		if ($count === false) {
+			$count = 0;
+		}
+		$result->closeCursor();
+		return (int)$count;
 	}
 
 	/**
